require 'pathname'
require 'fileutils'

# Class for explicit login and logout of user.
#
# <tt><b>author</b></tt> uni430
#
# <tt><b>since</b></tt> UES7-02
class Ues::Platform::Security::Authn::UESLogin

  private

  # Path to password files
  PASSWORD_FILES_PATH_PARAM = 'ues.platform.security.authn.password-files-dir' #:nodoc:

  public

  # Logs user in. As credentials are accepted:
  # * Empty or nil parameter (invokes login dialog on client side)
  # * Full path to file containing credentials (must be in form of property file containing keys accessCode1, accessCode2 and optional securityRealm)
  # * Relative path to file (in UES home) containing credentials (must be in form of property file containing keys accessCode1, accessCode2 and optional securityRealm)
  # * Access Code 1 and Access Code 2
  # * Security Realm, Access Code 1 and Access Code 2
  #
  # <tt><b>author</b></tt> uni430
  #
  # <tt><b>since</b></tt> UES7-02
  def self.login(*credentials)
    authnSvc = Java::cz.ues.platform.container.UESComponentFactory.getComponent(Java::cz.ues.platform.security.authn.UESAuthenticationService.java_class)
    if ((credentials == nil) || (credentials.size == 0))
      authnSvc.authenticate(nil)
    elsif (credentials.size == 1)
      crd = loadCredentials(credentials[0])
      if ((!crd.has_key?("accessCode1")) && (Java::cz.ues.platform.container.UESComponentFactory.containsComponent(Java::cz.ues.platform.security.authn.PasswordFileAuthenticationService.java_class)))
        authnSvc = Java::cz.ues.platform.container.UESComponentFactory.getComponent(Java::cz.ues.platform.security.authn.PasswordFileAuthenticationService.java_class)
        authnSvc.authenticate(credentials[0])
      else
        if (crd["securityRealm"] == nil)
          authnSvc.authenticate(Java::cz.ues.platform.security.authn.token.UESPasswordAuthnToken.new(crd["accessCode1"], crd["accessCode2"]))
        else
          authnSvc.authenticate(Java::cz.ues.platform.security.authn.token.UESPasswordAuthnToken.new(crd["securityRealm"], crd["accessCode1"], crd["accessCode2"]))
        end
      end
    elsif (credentials.size == 2)
      authnSvc.authenticate(Java::cz.ues.platform.security.authn.token.UESPasswordAuthnToken.new(credentials[0], credentials[1]))
    elsif (credentials.size == 3)
      authnSvc.authenticate(Java::cz.ues.platform.security.authn.token.UESPasswordAuthnToken.new(credentials[0], credentials[1], credentials[2]))
    else
      raise "Given parameters cannot be used for login."
    end
  end

  # Checks if we are already logged in.
  #
  # <tt><b>author</b></tt> uni430
  #
  # <tt><b>since</b></tt> UES7-02
  def self.isLoggedIn
    authnCtx = Java::cz.ues.platform.container.UESComponentFactory.getComponent(Java::cz.ues.platform.security.authn.internal.AuthenticationContext.java_class)
    # We must use authentication context because isAuthenticated method on UES Authentication service
    # invokes silent authentication which results in showing login dialog on client
    return ((authnCtx.isAuthenticated != nil) && (authnCtx.isAuthenticated))
  end

  class << Ues::Platform::Security::Authn::UESLogin
    alias logged_in? isLoggedIn
  end

  # Logout user.
  #
  # <tt><b>author</b></tt> uni430
  #
  # <tt><b>since</b></tt> UES7-02
  def self.logout
    authnCtx = Java::cz.ues.platform.container.UESComponentFactory.getComponent(Java::cz.ues.platform.security.authn.internal.AuthenticationContext.java_class)
    procMng = Java::cz.ues.platform.container.UESComponentFactory.getComponent(Java::cz.ues.platform.commons.pcb.UESProcessManager.java_class)
    loggedPrincipal = authnCtx.getPrincipal(Java::cz.ues.platform.security.authn.principal.UESPrincipal.java_class)
    activePrincipal = loggedPrincipal
    while (logged_in? && loggedPrincipal.equals(activePrincipal))
      procMng.exitProcess
      activePrincipal = authnCtx.getPrincipal(Java::cz.ues.platform.security.authn.principal.UESPrincipal.java_class)
    end
    if (Java::cz.ues.platform.essentials.reflection.ClassHelper.getClass("cz.unicorn.ues_v5.af_v1.aaa_v1.aute_v2.AuthenticationService"))
      auts = Java::cz.ues.platform.container.UESComponentFactory.getComponent(Java::cz.unicorn.ues_v5.af_v1.aaa_v1.aute_v2.AuthenticationService.java_class)
      auts.logout
      accessRole = authnCtx.getPrincipal(Java::cz.ues.platform.security.authn.principal.AccessRolePrincipal.java_class)
      if (accessRole != nil)
        auts.login(Java::cz.unicorn.ues_v5.af_v1.aaa_v1.aute_v2.spi.PrincipalCallbackHandler.new(accessRole.getName));
      end
    end
  end

  def self.loadCredentials(path) #:nodoc:
    fullPath = nil
    if (Pathname.new(path).absolute?)
      fullPath = path
    else
      fullPath = File.join(getPasswordFilesDir, path)
    end
    if (!File.exist?(fullPath))
      raise "File with credentials #{fullPath} does not exist."
    end
    content = binread_file(fullPath)
    if (Java::cz.ues.platform.security.authn.internal.SimpleEncryptionTool.isEncrypted(content))
      content = Java::cz.ues.platform.security.authn.internal.SimpleEncryptionTool.decrypt(content)
    end
    credentials = parseProperties(content)
    loadTrustedCredentials(credentials)
  end
  private_class_method :loadCredentials

  def self.loadTrustedCredentials(credentials) #:nodoc:
    securityRealm = credentials["securityRealm"]
    if (securityRealm.nil?)
      return credentials
    end
    scriptUri = UESScriptContext.getScriptUri()
    if (scriptUri.nil?)
      return credentials
    end
    if (Java::cz.ues.platform.essentials.reflection.ClassHelper.getClass("cz.ues.platform.security.appsecret.internal.ScriptAppSecretProvider").nil?)
      return credentials
    end
    secProv = Java::cz.ues.platform.container.UESComponentFactory.getComponent(Java::cz.ues.platform.security.appsecret.internal.ScriptAppSecretProvider.java_class)
    if (securityRealm != secProv.getUntrustedRealm().to_s)
      return credentials
    end
    appSecret = secProv.getAppSecret(scriptUri.to_s, secProv.getTrustedRealm())
    if (appSecret.nil?)
      return credentials
    end
    credentials["securityRealm"] = secProv.getTrustedRealm().to_s
    credentials["accessCode2"] << appSecret.to_s
    return credentials
  end
  private_class_method :loadTrustedCredentials

  def self.getPasswordFilesDir #:nodoc:
    # First try to read directory from system property
    result = Java::java.lang.System.getProperty(PASSWORD_FILES_PATH_PARAM)
    # If not set read directory from configuration
    if result.nil?
      cfg_file_path = File.join(Java::cz.ues.platform.container.UESSystem.getUesHome, 'config/ues-client.properties')
      if File.exist?(cfg_file_path)
        result = parseProperties(binread_file(cfg_file_path))[PASSWORD_FILES_PATH_PARAM]
      end
    end
    # If not set, return UES Home as default path
    if result.nil?
      result = Java::cz.ues.platform.container.UESSystem.getUesHome
    end
    return result
  end
  private_class_method :getPasswordFilesDir

  def self.parseProperties(data) #:nodoc:
    # Based on http://snippets.dzone.com/posts/show/1311
    properties = {}
    data.gsub!(/\r\n?/, "\n")
    data.each_line do |line|
      line.strip!
      if (line[0] != ?# and line[0] != ?=)
        i = line.index('=')
        if (i)
          properties[line[0..i - 1].strip] = line[i + 1..-1].strip
        else
          properties[line] = ''
        end
      end
    end
    properties
  end
  private_class_method :parseProperties

  # Based on the Ruby Best Practices book
  if RUBY_VERSION < '1.9'
    def self.ruby_18
      yield
    end
    private_class_method :ruby_18
    
    def self.ruby_19
      false
    end
    private_class_method :ruby_19
  else
    def self.ruby_18
      false
    end
    private_class_method :ruby_18
    
    def self.ruby_19
      yield
    end
    private_class_method :ruby_19
  end
  
  def self.binread_file(file_path)
    ruby_18 { File.open(file_path, 'rb') { |file| file.read } } ||
      ruby_19 {File.binread(file_path)}
  end
  private_class_method :binread_file

end
